@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -227,7 +227,7 @@ nd_ip_set_gui(const LND_Packet *packet, LND_ProtoInfo *pinf)
   nd_ip_set_gui_off(pinf, iphdr);
   nd_ip_set_gui_ttl(pinf, iphdr);
   nd_ip_set_gui_p(pinf, iphdr);
-  nd_ip_set_gui_sum(pinf, iphdr);
+  nd_ip_set_gui_sum(packet, pinf, iphdr);
   nd_ip_set_gui_src(pinf, iphdr);
   nd_ip_set_gui_dst(pinf, iphdr);
   nd_ip_set_gui_options(pinf, iphdr, packet);
@@ -347,10 +347,23 @@ nd_ip_set_gui_p(LND_ProtoInfo *pinf, struct ip *iphdr)
 
 
 void       
-nd_ip_set_gui_sum(LND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_sum(const LND_Packet *packet, LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[12], DATA_TO_PTR(ntohs(iphdr->ip_sum)));
   
+  /* Checksum highlighting -- if we're beyond the point in
+   * the packet where it makes sense to check this, we 
+   * show a "maybe", otherwise we check ourselves.
+   */
+  if (libnd_packet_get_proto_index(packet, &pinf->inst) >
+      libnd_packet_get_last_fixable_proto(packet))
+    {
+      nd_proto_info_field_set_state(pinf, 
+				    &ip_fields[12],
+				    ND_FIELD_STATE_UNKN);
+      return;
+    }
+
   if (!libnd_ip_csum_correct(iphdr, NULL))
     nd_proto_info_field_set_state(pinf, 
 				  &ip_fields[12],
