@@ -67,7 +67,7 @@ void       nd_ip_set_gui_mf(LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_off(LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_ttl(LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_p(LND_ProtoInfo *pinf, struct ip *iphdr);
-void       nd_ip_set_gui_sum(LND_ProtoInfo *pinf, struct ip *iphdr);
+void       nd_ip_set_gui_sum(const LND_Packet *packet, LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_src(LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_dst(LND_ProtoInfo *pinf, struct ip *iphdr);
 void       nd_ip_set_gui_options(LND_ProtoInfo *pinf, struct ip *iphdr,
