@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -265,7 +265,12 @@ nd_tcp_set_gui_cksum(LND_ProtoInfo *pinf, struct ip *iphdr,
 {
   nd_proto_field_set(pinf, &tcp_fields[13], DATA_TO_PTR(ntohs(tcphdr->th_sum)));
 
-  if ((guchar *)iphdr + ntohs(iphdr->ip_len) > libnd_packet_get_end(packet))
+  /* Show as "unknown" if we don't have the whole payload
+   * or we somehow cannot know.
+   */
+  if ((guchar *)iphdr + ntohs(iphdr->ip_len) > libnd_packet_get_end(packet) ||
+      libnd_packet_get_proto_index(packet, &pinf->inst) >
+      libnd_packet_get_last_fixable_proto(packet))
     {
       nd_proto_info_field_set_state(pinf,
 				    &tcp_fields[13],
