@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,9 @@ IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <nd_tcp.h>
 #include <nd_tcp_callbacks.h>
@@ -148,7 +151,7 @@ nd_tcp_ack_cb(LND_Packet  *packet,
 
   nd_dialog_number(_("Enter acknowledgement number:"),
 		   ND_BASE_DEC,
-		   ntohl(tcphdr->th_seq), (guint) -1,
+		   ntohl(tcphdr->th_ack), (guint) -1,
 		   tcp_ack_ok_cb,
 		   NULL, packet, data);
 
