@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -711,6 +711,8 @@ on_protos_notebook_switch_page         (GtkNotebook     *notebook,
   ND_Trace         *trace_gui;
   LND_ProtoInfo    *pinf;
   LND_ProtoData    *pd;
+  LND_ProtoInst     pi, *pi_ptr;
+  LND_Protocol     *pcap_proto;
 
   trace = gtk_object_get_data(GTK_OBJECT(notebook), "trace");
   trace_gui = nd_trace_get(trace);
@@ -727,16 +729,32 @@ on_protos_notebook_switch_page         (GtkNotebook     *notebook,
   if (!pinf)
     return;
 
-  pd = libnd_packet_get_proto_data(trace_gui->cur_packet, pinf->inst.proto, pinf->inst.nesting);
-  nd_trace_set_current_proto_selection(trace, &pd->inst);
-  nd_trace_set_current_proto(trace, &pd->inst, TRUE);
+  /* Special case for pcap tab: libnd_packet_get_proto_data()
+   * won't find information on this tab, so we check if that's
+   * the desired protocol and bypass the "real" lookup in the
+   * packet.
+   */
+  pcap_proto = libnd_proto_registry_find(LND_PROTO_LAYER_LINK, 0xa1b2c3d4);
+
+  if (pcap_proto && pinf->inst.proto == pcap_proto)
+    {
+      libnd_proto_inst_init(&pi, pcap_proto, 0);
+      pi_ptr = &pi;
+    }
+  else
+    {
+      pd = libnd_packet_get_proto_data(trace_gui->cur_packet, pinf->inst.proto, pinf->inst.nesting);
+      pi_ptr = &pinf->inst;
+    }
+
+  nd_trace_set_current_proto_selection(trace, pi_ptr);
+  nd_trace_set_current_proto(trace, pi_ptr, TRUE);
   
   return;
   TOUCH(page_num);
   TOUCH(user_data);
 }
 
-
 void
 on_file_save_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
