@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -628,7 +628,7 @@ hex_update_current(GtkHex *hex, int byte_offset)
 {
   GtkEditable *ed;
   int  old, old2, i1, i2;
-  guchar s[16];
+  char s[16];
 
   if (!hex || hex->mode != GTK_RAW_HEX)
     return;
@@ -1092,8 +1092,8 @@ hex_get_hex_text(const guchar *data, int data_size)
   const guchar *data_ptr;
   guchar      *hex_data_ptr;
   guchar      *ascii_ptr;
-  guchar       hex_byte[3];
-  guchar       offset[5];
+  char         hex_byte[3];
+  char         offset[5];
   guchar       ascii_byte;
 	
   if (!data)
@@ -1452,7 +1452,7 @@ gtk_hex_new(void)
 
 
 void       
-gtk_hex_set_content(GtkHex *hex, char *data, int data_size)
+gtk_hex_set_content(GtkHex *hex, guchar *data, int data_size)
 {
   guchar     *hex_text = NULL;
   guchar     *null_map_text = NULL;
@@ -1463,18 +1463,18 @@ gtk_hex_set_content(GtkHex *hex, char *data, int data_size)
   switch (hex->mode)
     {
     case GTK_RAW_HEX:
-      hex_text = hex_get_hex_text(data, data_size);
+      hex_text = hex_get_hex_text((guchar *) data, data_size);
       if (!hex_text)
 	goto cleanup;
       break;
 
     case GTK_RAW_ASCII:
-      hex_get_ascii_text(data, data_size, &hex_text, &null_map_text);
+      hex_get_ascii_text((guchar *) data, data_size, &hex_text, &null_map_text);
       if (!hex_text || !null_map_text)
 	goto cleanup;
 
       null_map_delete(hex->priv, 0, -1);
-      null_map_insert(hex->priv, null_map_text, 0, strlen(null_map_text));
+      null_map_insert(hex->priv, (char *) null_map_text, 0, strlen((char *) null_map_text));
       break;
       
     default:
@@ -1486,13 +1486,13 @@ gtk_hex_set_content(GtkHex *hex, char *data, int data_size)
 
   gtk_editable_delete_text(GTK_EDITABLE(hex->hex_text), 0, -1);
   gtk_text_insert(GTK_TEXT(hex->hex_text), NULL, NULL, NULL,
-		  hex_text, strlen(hex_text));
+		  (char *) hex_text, strlen((char *) hex_text));
 
   gtk_signal_handler_unblock_by_data(GTK_OBJECT(hex->hex_text), hex);
   gtk_text_thaw(GTK_TEXT(hex->hex_text));
 
   /* Old data isn't freed -- we don't possess it! */
-  hex->data = data;
+  hex->data = (guchar *) data;
   hex->data_size = data_size;
 
  cleanup:
