@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -80,7 +80,7 @@ struct _GtkHexClass
 
 guint      gtk_hex_get_type(void);
 GtkWidget *gtk_hex_new(void);
-void       gtk_hex_set_content(GtkHex *hex, char *data, int data_size);
+void       gtk_hex_set_content(GtkHex *hex, guchar *data, int data_size);
 void       gtk_hex_set_mode(GtkHex *hex, GtkRawMode mode);
 
 #ifdef __cplusplus
