@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -1198,18 +1198,22 @@ nd_dialog_about(void)
   static GtkWidget *dialog = NULL;
   static char *about_text =N_("\n"
 			      "   Programming:\n"
-			      "     Christian Kreibich <christian.kreibich-AT-cl.cam.ac.uk>\n"
+			      "     Christian Kreibich <christian@whoop.org>\n"
 			      "\n"
-			      "   Help, Feedback,\n"
-			      "   Suggestions, Inspiration:\n"
+			      "   Help, feedback,\n"
+			      "   suggestions, inspiration:\n"
 			      "     Mark Handley\n"
 			      "     Vern Paxson\n"
 			      "     Daniel Stodden\n"
 			      "     Euan Harris\n"
 			      "     Andrew Moore\n"
 			      "\n"
-			      "   Yet Another SRG Production\n"
-			      "   http://www.cl.cam.ac.uk/Research/SRG\n\n"
+			      "   Some artwork provided by the Tango project.\n"
+			      "     http://tango.freedesktop.org/\n"
+			      "     http://creativecommons.org/licenses/by-sa/2.5/\n"
+			      "\n"
+			      "   Yet another ICIR production -- http://www.icir.org/\n"
+			      "\n"
 			      "   ");
 
   GtkWidget *label;
