@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -1095,6 +1095,75 @@ nd_gui_list_clear(LND_Trace *trace)
 }
 
 
+static gboolean      
+gui_list_update_packet(const LND_Packet *packet, char *line, int len)
+{
+  int do_magic = 0;
+  guchar *raw_start, *raw_end;
+  
+  if (! libnd_tcpdump_get_packet_line(packet, line, len, TRUE))
+    {
+      nd_dialog_message(_("Tcpdump I/O error"),
+			_("An error occurred when obtaining tcpdump packet output.\n"
+			  "Not all packet ouput has been updated."),
+			TRUE);
+      return FALSE;
+    }
+  
+  libnd_prefs_get_int_item(ND_DOM_NETDUDE, "show_magic_result", &do_magic);
+  
+  if (!do_magic)
+    return TRUE;
+  
+  raw_start = libnd_packet_get_data(packet, libnd_raw_proto_get(), 0);
+  raw_end = libnd_packet_get_data_end(packet, libnd_raw_proto_get(), 0);
+
+  if (raw_start && raw_end)
+    {
+      char magicstr[512];
+      guchar *raw_start2 = NULL;
+      guchar *sep = strstr(raw_start, "\r\n\r\n");
+      
+      if (sep && sep + 4 != raw_end)
+	{
+	  char *m1, *m2;
+	  raw_start2 = sep + 4;
+	  
+	  m1 = strdup(libnd_magic_buffer(raw_start, raw_start2 - raw_start));
+	  m2 = strdup(libnd_magic_buffer(raw_start2, raw_end - raw_start2));
+	  
+	  if (m1[0] != '\0' || m2[0] != '\0')
+	    {
+	      snprintf(magicstr, 512, " [%s%s%s]",
+		       m1[0] != '\0' ? m1 : "",
+		       m1[0] != '\0' && m2[0] != '\0' ? "/" : "",
+		       m2[0] != '\0' ? m2 : "");
+	      magicstr[511] = '\0';
+	      strncat(line, magicstr, len - strlen(line));
+	      line[len-1] = '\0';
+	    }
+	  
+	  g_free(m1);
+	  g_free(m2);
+	}
+      else
+	{
+	  const char *magic = libnd_magic_buffer(raw_start, raw_end - raw_start);
+	  
+	  if (magic[0] != '\0')
+	    {
+	      snprintf(magicstr, 512, " [%s]", magic);
+	      magicstr[511] = '\0';
+	      strncat(line, magicstr, len - strlen(line));
+	      line[len-1] = '\0';
+	    }
+	}
+    }
+  
+  return TRUE;
+}
+
+
 void    
 nd_gui_list_update(LND_Trace *trace)
 {
@@ -1126,14 +1195,8 @@ nd_gui_list_update(LND_Trace *trace)
   
   for (packet = libnd_tpm_get_packets(trace->tpm), i = 0; packet; packet = packet->next, i++)
     {
-      if (! libnd_tcpdump_get_packet_line(packet, line, MAXPATHLEN, TRUE))
-	{
-	  nd_dialog_message(_("Tcpdump I/O error"),
-			    _("An error occurred when obtaining tcpdump packet output.\n"
-			      "Not all packet ouput has been updated."),
-			    TRUE);
-	  break;
-	}
+      if (! gui_list_update_packet(packet, line, MAXPATHLEN))
+	break;
       
       gtk_clist_append(clist, s);      
       nd_gui_list_set_row_incomplete(trace, i, ! libnd_packet_is_complete(packet));
@@ -1162,17 +1225,9 @@ nd_gui_list_update_packet(const LND_Packet *packet)
     return;
 
   index = libnd_packet_get_index(packet);
-
-  if (! libnd_tcpdump_get_packet_line(packet, line, MAXPATHLEN, TRUE))
-    {
-      g_snprintf(line, MAXPATHLEN, "[tcpdump I/O error]");
-      nd_dialog_message(_("Tcpdump I/O error"),
-			_("An error occurred when obtaining tcpdump packet output.\n"),
-			TRUE);
-    }
-
+  gui_list_update_packet(packet, line, MAXPATHLEN);
   gtk_clist_set_text(GTK_CLIST(trace_gui->list), index, 0, line);
-
+  
   nd_gui_list_set_row_incomplete(libnd_packet_get_trace(packet), index,
 				 ! libnd_packet_is_complete(packet));
 }
@@ -1301,7 +1356,7 @@ nd_gui_list_select_all(LND_Trace *trace, gboolean emit_signal)
 }
 
 
-static void
+static gboolean
 gui_update_line_cb(LND_Packet *packet,
 		   LND_ProtoData *pd,
 		   void *user_data)
@@ -1309,10 +1364,12 @@ gui_update_line_cb(LND_Packet *packet,
   char *line = (char *) user_data;
 
   if (!packet || !pd)
-    return;
+    return TRUE;
   
   if (pd->inst.proto->is_stateful)
     pd->inst.proto->update_tcpdump_line(packet, line);
+
+  return TRUE;
 }
 
 
@@ -1511,7 +1568,6 @@ nd_gui_trace_new_tab(LND_Trace *trace)
                       GTK_SIGNAL_FUNC (on_protos_notebook_switch_page),
                       notebook);
 
-
   gtk_object_set_data(GTK_OBJECT(tab_content), "trace", trace);
   gtk_object_set_data(GTK_OBJECT(notebook), "trace", trace);
 
