@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -127,7 +127,8 @@ nd_packet_init(void)
   ob->packet_modified       = packet_ob_fix_gui;
   ob->packet_updated        = packet_ob_fix_gui;
   ob->packet_fixed          = packet_ob_fix_gui;
-
+  /* Packet len/caplen change will trigger packet_fixed anyway */
+  
   libnd_packet_add_observer(ob);
 }
 
@@ -179,6 +180,14 @@ nd_packet_set_filtered_state(LND_Packet *packet)
   D_RETURN;
 }
 
+static gboolean
+packet_set_gui_cb(LND_Packet *packet, LND_ProtoData *pd, void *user_data)
+{
+  LND_Trace *trace = (LND_Trace *) user_data;
+  nd_trace_add_proto_tab(trace, pd->inst.proto, pd->inst.nesting);
+  return TRUE;
+  TOUCH(packet);
+}
 
 void    
 nd_packet_set_gui(const LND_Packet *packet)
@@ -188,10 +197,11 @@ nd_packet_set_gui(const LND_Packet *packet)
   ND_Protocol    *proto_gui;
   ND_Trace       *trace_gui;
   LND_Trace      *trace;
-  LND_Protocol   *raw_proto;
+  LND_Protocol   *raw_proto, *pcap_proto = NULL;
   LND_ProtoData  *pd;
   LND_ProtoInfo  *pinf;
   int             i;
+  int             real_proto_start = 0; /* the tab at which "real" protocols start */
   GList          *l;
   gboolean        has_sel_proto = FALSE;
 
@@ -199,6 +209,13 @@ nd_packet_set_gui(const LND_Packet *packet)
 
   D(("Setting gui to packet %p\n", packet));
 
+  /* Whatever we do in this function does not affect the user's preferred
+   * protocol, so make sure we take no switching of the current notebook
+   * tabs as the signal to update that preferential protocol. We do this
+   * by unhooking the signal handler that takes care of updating this
+   * preferential choice.
+   */  
+
   if (!packet)
     {
       LND_Trace *trace;
@@ -206,18 +223,18 @@ nd_packet_set_gui(const LND_Packet *packet)
       return_if_no_current_trace(trace);
       notebook = nd_trace_get_notebook(trace);
       D_ASSERT_PTR(notebook);
-      
-      /* Hide all tabs */
+     
+      /* Hide all tabs */      
+      i = 0;
       page = gtk_notebook_get_nth_page(notebook, 0);
       
-      for (i = 0; gtk_notebook_get_nth_page(notebook, i); i++)
+      while (page)
 	{
-	  page = gtk_notebook_get_nth_page(notebook, i);
-	  D_ASSERT_PTR(page);
 	  gtk_widget_hide(page);
+	  page = gtk_notebook_get_nth_page(notebook, ++i);
 	}
-
-      D_RETURN;
+      
+      goto flush;
     }
 
   trace = libnd_packet_get_trace(packet);
@@ -227,20 +244,40 @@ nd_packet_set_gui(const LND_Packet *packet)
   notebook = nd_trace_get_notebook(trace);
   D_ASSERT_PTR(notebook);
 
-  /* Show and hide the protocol tabs for this packet,
-     and while doing this, also gray out the protocol menus
-     that aren't applicable for this packet:
-  */
-  page = gtk_notebook_get_nth_page(notebook, 0);
+  /* See if we have the pcap "protocol" plugin; we always should. */
+  pcap_proto = libnd_proto_registry_find(LND_PROTO_LAYER_LINK, 0xa1b2c3d4);
 
+  /* First ensure that the notebook actually has tabs for all the
+   * protocols contained in the packet. After that, we hide any
+   * tabs for the protocols in the notebook that are not in the packet.
+   * We can just attempt to add new tabs via nd_trace_add_proto_tab(),
+   * since it'll detect when a protocol is already registered and return.
+   */
+  libnd_packet_foreach_proto((LND_Packet*) packet, packet_set_gui_cb, trace);
+
+  /* Now show/hide the protocol tabs for this packet, and also gray
+   * out the protocol menus that aren't applicable for this packet.
+   */
+  page = gtk_notebook_get_nth_page(notebook, 0);
+  
   for (i = 0; gtk_notebook_get_nth_page(notebook, i); i++)
-    {
+    {  
       page = gtk_notebook_get_nth_page(notebook, i);
       D_ASSERT_PTR(page);
 
       ND_GTK_GET(pinf, page, "pinf");
       proto_gui = nd_proto_get(pinf->inst.proto);
       
+      /* Always show the pcap "protocol" if we have it. */
+      if (pcap_proto && pinf->inst.proto == pcap_proto)
+	{
+	  gtk_notebook_reorder_child(notebook, nd_proto_info_get(pinf)->proto_tab, 0);
+	  proto_gui->set_gui(packet, pinf);
+	  real_proto_start = 1;
+	  gtk_widget_show(page);
+	  continue;
+	}
+      
       /* Whereas for the tab itself, we want to keep an
        * eye on that nesting level.
        */
@@ -255,7 +292,11 @@ nd_packet_set_gui(const LND_Packet *packet)
 	}
     }
 
-  if (trace_gui->cur_pi_sel)
+  /* Set has_sel_proto to TRUE if the packet has the currently
+   * selected protocol. This is so we can try to keep that
+   * protocol selected for the new packet.
+   */
+  if (trace_gui->cur_pi_sel && trace_gui->cur_pi_sel->proto != pcap_proto)
     {
       has_sel_proto =
 	libnd_packet_has_proto_nested(packet,
@@ -274,9 +315,10 @@ nd_packet_set_gui(const LND_Packet *packet)
       gtk_label_set_text(GTK_LABEL(pinf_raw->proto_label), raw_proto->name);
     }
 
-  /* Now, update each of the packet's protocol header's contents
-     in the GUI: */
-  for (l = packet->pd, i = 0; l; l = g_list_next(l), i++)
+  /* Now, update each of the packet's protocol header's contents in the GUI,
+   * and make sure they're sorted correctly.
+   */
+  for (l = packet->pd, i = real_proto_start; l; l = g_list_next(l), i++)
     {
       pd = (LND_ProtoData *) l->data;
       
@@ -299,16 +341,32 @@ nd_packet_set_gui(const LND_Packet *packet)
       proto_gui->set_gui(packet, pinf);
       nd_gui_proto_table_unblock_events(trace, pinf);
 
-      /* sort the tabs so that the GUI reflects the sequence in
-	 the packet: */
-      
       gtk_notebook_reorder_child(notebook, nd_proto_info_get(pinf)->proto_tab, i);
     }
-
+  
+  gtk_signal_handler_block_by_data(GTK_OBJECT(notebook), notebook);
   if (trace_gui->cur_pi_sel)
     nd_trace_set_current_proto(trace, trace_gui->cur_pi_sel, TRUE);
+  gtk_signal_handler_unblock_by_data(GTK_OBJECT(notebook), notebook);
 
   nd_gui_list_update_packet(packet);    
-
+  
+ flush:
+  
+  /* Okay, this one took forever. The problem is that we do NOT want
+   * any protocol tab switches in the notbook to be understood as user
+   * input here (which would cause the user's preferred protocol to be
+   * switched, likely to the pcap tab). Since all the relevant signal
+   * handlers normally are triggered *after* we leave this function,
+   * we force an event flugh at this point, so we can make sure we
+   * block the signal handler that'd update the preferred-protocol
+   * selection.
+   */
+  
+  gtk_signal_handler_block_by_data(GTK_OBJECT(notebook), notebook);
+  while(gtk_events_pending())
+    gtk_main_iteration();
+  gtk_signal_handler_unblock_by_data(GTK_OBJECT(notebook), notebook);
+  
   D_RETURN;
 }
