@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -326,7 +326,30 @@ nd_prefs_dialog_sync(void)
   libnd_prefs_foreach_domain(prefs_dialog_sync_cb, NULL);
 }
 
+static void
+prefs_update_packets_cb(LND_Trace *trace, void *user_data)
+{
+  libnd_tcpdump_close(trace);
+  libnd_tcpdump_open(trace);
+  
+  nd_gui_list_update(trace);
+}
 
+static void
+prefs_update_cb(LND_PrefsDomain *domain, void *user_data)
+{
+  LND_Trace      *trace;
+  
+  if ( (trace = nd_trace_registry_get_current()))
+    nd_gui_set_windowtitle(libnd_trace_get_name(trace));
+  
+  nd_gui_update_monowidth_widgets();
+  nd_trace_registry_foreach(prefs_update_packets_cb, NULL);
+  
+  return;
+  TOUCH(domain);
+  TOUCH(user_data);
+}
 
 /* This is our local callback for updating anything in the local
  * config domain to new settings. Gets called on startup and whenever
@@ -335,15 +358,8 @@ nd_prefs_dialog_sync(void)
 static void
 prefs_apply_cb(LND_PrefsDomain *domain, void *user_data)
 {
-  LND_Trace      *trace;
-
   prefs_sync_settings_to_dialog();
 
-  if ( (trace = nd_trace_registry_get_current()))
-    nd_gui_set_windowtitle(libnd_trace_get_name(trace));
-
-  nd_gui_update_monowidth_widgets();
-
   return;
   TOUCH(domain);
   TOUCH(user_data);
@@ -368,6 +384,7 @@ nd_prefs_init(void)
 				  sizeof (prefs_entries_netdude) / sizeof(LND_PrefsEntry));
 
   libnd_prefs_domain_add_apply_cb(domain, prefs_apply_cb);
+  libnd_prefs_domain_add_update_cb(domain, prefs_update_cb);
 
   libnd_prefs_save();
 
