@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,9 @@ IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <gtkhex.h>
 
@@ -115,10 +118,10 @@ raw_create_gui(LND_Trace *trace, LND_ProtoInfo *pinf)
 static void       
 raw_set_gui(const LND_Packet *packet, LND_ProtoInfo *pinf)
 {
-  guchar *data     = libnd_packet_get_data(packet, libnd_raw_proto_get(), pinf->inst.nesting);
+  guchar *data = libnd_packet_get_data(packet, libnd_raw_proto_get(), pinf->inst.nesting);
   guchar *data_end = libnd_packet_get_data_end(packet, libnd_raw_proto_get(), pinf->inst.nesting);
   GtkHex *hex;
-  
+
   if (!pinf)
     {
       D(("Protocol info for raw at %i not found\n", pinf->inst.nesting));
@@ -126,8 +129,11 @@ raw_set_gui(const LND_Packet *packet, LND_ProtoInfo *pinf)
     }
   
   hex = libnd_reg_get_data(pinf->registry, ND_RAW_HEX_KEY);
-  
-  if (data != hex->data)
+
+  /* Only if the data really change, do we update. When updating, we
+   * lose the cursor location, so we must avoid unnecessary updates.
+   */
+  if (data != hex->data || data_end - data != (int) hex->data_size)
     gtk_hex_set_content(hex, data, data_end - data);
 }
 
