@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2007 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -394,14 +394,13 @@ nd_trace_set_current_proto_selection(LND_Trace *trace,
   if (!trace_gui->cur_pi_sel)
     {
       trace_gui->cur_pi_sel = g_new0(LND_ProtoInst, 1);
-
+      
       if (!trace_gui->cur_pi_sel)
 	return;
     }
 
   D(("Setting current proto selection to %s/%i\n",
      pi->proto->name, pi->nesting));
-
   *trace_gui->cur_pi_sel = *pi;
 }
 
@@ -413,37 +412,41 @@ nd_trace_set_current_proto(LND_Trace *trace,
 {
   ND_Trace    *trace_gui = nd_trace_get(trace);
   GtkNotebook *notebook;
+  LND_Packet *packet = NULL;
   int num;
 
-  if (!trace_gui || !trace_gui->cur_packet || !pi)
-    return;
-
-  D(("Setting current proto to %s/%i\n",
-     (pi ? pi->proto->name : "none"),
-     (pi ? pi->nesting : 0)));
-
+  if (! trace_gui || ! trace_gui->cur_packet || ! pi ||
+      ! (packet = trace_gui->cur_packet) ||
+      ! libnd_packet_has_proto_nested(packet, pi->proto, pi->nesting))
+    {
+      if (gui_update)
+	nd_gui_statusbar_clear(NULL);
+      return;
+    }
+  
+  D(("Setting current proto to %s/%i\n", pi->proto->name, pi->nesting));
+  
   if (!trace_gui->cur_pi)
     {
-      trace_gui->cur_pi = g_new0(LND_ProtoInst, 1);
-
+      trace_gui->cur_pi = g_new0(LND_ProtoInst, 1);      
       if (!trace_gui->cur_pi)
 	return;
     }
   
   *trace_gui->cur_pi = *pi;
-
-  if (trace_gui->cur_packet)
+  
+  if (packet)
     {
-      trace_gui->cur_header = libnd_packet_get_data(trace_gui->cur_packet,
-						    pi->proto,
-						    pi->nesting);
+      trace_gui->cur_header = libnd_packet_get_data(packet, pi->proto, pi->nesting);
       D(("NEW current header: %p\n", trace_gui->cur_header));
     }
-
+  
   if (gui_update)
     {
+      LND_Protocol *pcap_proto = libnd_proto_registry_find(LND_PROTO_LAYER_LINK, 0xa1b2c3d4);
       LND_ProtoInfo *pinf;
       ND_ProtoInfo  *pinf_gui;
+      char buf[512];
 
       notebook = nd_trace_get_notebook(trace);
       pinf = nd_trace_get_proto_info(trace, pi->proto, pi->nesting);
@@ -459,6 +462,26 @@ nd_trace_set_current_proto(LND_Trace *trace,
       if (num != gtk_notebook_get_current_page(notebook))
 	gtk_notebook_set_page(notebook, num);
       gtk_signal_handler_unblock_by_data(GTK_OBJECT(notebook), notebook);
+
+      /* Report the protocol's size in bytes in the statusbar if feasible. */
+      if (! pi->proto || (pcap_proto && pi->proto == pcap_proto))
+	nd_gui_statusbar_clear(NULL);
+      else
+	{
+	  /* Find out the index of the given protocol instance, and use it
+	   * to get the next one. We compute the difference in data start pointers
+	   * to obtain the size of the displayed data chunk.
+	   */
+	  int proto_index = libnd_packet_get_proto_index(packet, pi);
+	  LND_ProtoData *pd = libnd_packet_get_nth_proto(packet, proto_index + 1);
+	  guchar *data_end = pd ? pd->data : libnd_packet_get_end(packet);
+	  
+	  g_snprintf(buf, 512, _("%i %s bytes shown."),
+		     data_end - libnd_packet_get_data(packet, pi->proto, pi->nesting),
+		     pi->proto->name);
+	  
+	  nd_gui_statusbar_set_noclear(buf);
+	}
     }
 }
 
@@ -467,32 +490,21 @@ void
 nd_trace_set_current_packet(LND_Trace *trace, LND_Packet *packet)
 {    
   ND_Trace *trace_gui = nd_trace_get(trace);
-  char      msg[MAXPATHLEN];
 
   D_ENTER;
 
   if (!trace_gui)
     D_RETURN;
 
-  msg[0] = '\0';
-
-  if (packet)
-    {
-      if (trace != libnd_packet_get_trace(packet))
-	D_RETURN;
-
-      g_snprintf(msg, MAXPATHLEN,
-		 _("Real size: %u bytes, captured: %u."),
-		 packet->ph.len, packet->ph.caplen);
-    }
-
-  nd_gui_statusbar_set_noclear(msg);
+  if (packet && trace != libnd_packet_get_trace(packet))
+    D_RETURN;
   
   D(("Current packet now %p\n", packet));
   trace_gui->cur_packet = packet;
-
+  
   nd_trace_set_current_proto(trace, trace_gui->cur_pi, FALSE);
   nd_packet_set_gui(packet);
+
   D_RETURN;
 }
 
